/*
 * E!Kit-1100 boards specific pcmcia routines.
 *
 * Copyright 2003-2004 Device Drivers Limited.
 * Author: kawamoto@devdrv.co.jp
 *
 * This program is largely copies by the Db1x00 source files.
 *
 * ########################################################################
 *
 *  This program is free software; you can distribute it and/or modify it
 *  under the terms of the GNU General Public License (Version 2) as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * ########################################################################
 *
 */
#include <linux/module.h>
#include <linux/init.h>
#include <linux/config.h>
#include <linux/delay.h>
#include <linux/ioport.h>
#include <linux/kernel.h>
#include <linux/tqueue.h>
#include <linux/timer.h>
#include <linux/mm.h>
#include <linux/proc_fs.h>
#include <linux/version.h>
#include <linux/types.h>

#include <pcmcia/version.h>
#include <pcmcia/cs_types.h>
#include <pcmcia/cs.h>
#include <pcmcia/ss.h>
#include <pcmcia/bulkmem.h>
#include <pcmcia/cistpl.h>
#include <pcmcia/bus_ops.h>
#include "cs_internal.h"

#include <asm/io.h>
#include <asm/irq.h>
#include <asm/system.h>

#include <asm/au1000.h>
#include <asm/au1000_pcmcia.h>

#include <asm/ekit1100.h>

static BCSR * const bcsr = (BCSR *)0xB8000000;
static unsigned short tps_reg = 0;

#define TPS_DATA_SET  outl(0x20000, SYS_OUTPUTSET)
#define TPS_DATA_CLR  outl(0x20000, SYS_OUTPUTCLR)
#define TPS_CLOCK_SET outl(0x40000, SYS_OUTPUTSET)
#define TPS_CLOCK_CLR outl(0x40000, SYS_OUTPUTCLR)
#define TPS_LATCH_SET outl(0x80000, SYS_OUTPUTSET)
#define TPS_LATCH_CLR outl(0x80000, SYS_OUTPUTCLR)
 
#define TPS_LENGTH 11

#define TPS_WAKE   0x100
#define TPS_AVCC   0x00C
#define TPS_AVCC0  0x000
#define TPS_AVCC3  0x004
#define TPS_AVCC5  0x008
#define TPS_BVCC   0x0C0
#define TPS_BVCC0  0x000
#define TPS_BVCC3  0x080
#define TPS_BVCC5  0x040

static void tps_write(unsigned short d)
{
	int i;

	tps_reg = d;

	TPS_DATA_CLR;
	TPS_CLOCK_CLR;
	TPS_LATCH_CLR;
	
	for(i = 0; i < TPS_LENGTH; i++)
	{
		if(d & (1 << (TPS_LENGTH - 1)))
			TPS_DATA_SET;
		else
			TPS_DATA_CLR;
		au_sync_udelay(1);
		
		TPS_CLOCK_SET;
		au_sync_udelay(1);
		TPS_CLOCK_CLR;
		au_sync_udelay(1);
		
		d <<= 1;
	}
	TPS_LATCH_SET;
	au_sync_udelay(1);
	TPS_CLOCK_SET;
	au_sync_udelay(1);
	TPS_CLOCK_CLR;
	au_sync_udelay(1);
	TPS_LATCH_CLR;
	au_sync_udelay(1);
}

static unsigned short tps_read(void)
{
	return tps_reg;
}

static int ekit1100_pcmcia_init(struct pcmcia_init *init)
{
	bcsr->cf0 = BCSR_CF0_RST;
	bcsr->cf1 = BCSR_CF1_RST;
	tps_write(TPS_WAKE | TPS_AVCC0 | TPS_BVCC0);
	au_sync_delay(5);

	return PCMCIA_NUM_SOCKS;
}

static int ekit1100_pcmcia_shutdown(void)
{
	bcsr->cf0 = BCSR_CF0_RST;
	bcsr->cf1 = BCSR_CF1_RST;
	tps_write(TPS_WAKE | TPS_AVCC0 | TPS_BVCC0);
	au_sync_delay(5);

	return 0;
}

static int ekit1100_pcmcia_socket_state(unsigned sock, struct pcmcia_state *state)
{
	u32 inserted;
	unsigned char vs;

	if(sock > PCMCIA_MAX_SOCK) return -1;

	state->ready  = 0;
	state->vs_Xv  = 0;
	state->vs_3v  = 0;
	state->detect = 0;

	if(sock == 0)
	{
		vs = (bcsr->cf0 & BCSR_CF0_VS) >> 2;
		inserted = (bcsr->cf0 & BCSR_CF0_CD);
	}
	else
	{
		vs = (bcsr->cf1 & BCSR_CF1_VS) >> 2;
		inserted = (bcsr->cf1 & BCSR_CF1_CD);
	}

	DEBUG(KERN_DEBUG "ekit1100 socket %d: inserted %d, vs %d\n", sock, inserted, vs); 

	if(inserted)
	{
		switch(vs)
		{
			case 0: /* 5V */
				break;
			case 1: /* 3V */
			case 3:
				state->vs_3v = 1;
				break;
			default:
				/* return without setting 'detect' */
				printk(KERN_ERR "ekit1100 bad VS (%d)\n", vs);
				return -1;
		}
		state->detect = 1;
		state->ready = 1;
	}
	else
	{
		/* if the card was previously inserted and then ejected,
		 * we should turn off power to it
		 */
		if((sock == 0) && (!(bcsr->cf0 & BCSR_CF0_RST)))
		{
			bcsr->cf0 = BCSR_CF0_RST;
			tps_write((tps_read() & ~TPS_AVCC) | TPS_AVCC0);
			au_sync_delay(5);
		}
		else if((sock == 1) && (!(bcsr->cf1 & BCSR_CF1_RST)))
		{
			bcsr->cf1 = BCSR_CF1_RST;
			tps_write((tps_read() & ~TPS_BVCC) | TPS_BVCC0);
			au_sync_delay(5);
		}
	}

	state->bvd1   = 1;
	state->bvd2   = 1;
	state->wrprot = 0;

	return 1;
}


static int ekit1100_pcmcia_get_irq_info(struct pcmcia_irq_info *info)
{
	if(info->sock > PCMCIA_MAX_SOCK) return -1;

	if(info->sock == 0)
		info->irq = AU1000_GPIO_0;
	else 
		info->irq = AU1000_GPIO_1;

	return 0;
}


static int ekit1100_pcmcia_configure_socket(const struct pcmcia_configure *configure)
{
	int sock = configure->sock;

	if(sock > PCMCIA_MAX_SOCK) return -1;

	DEBUG(KERN_DEBUG "socket %d Vcc %dV Vpp %dV, reset %d\n", 
			sock, configure->vcc, configure->vpp, configure->reset);

	/* pcmcia reg was set to zero at init time. Be careful when
	 * initializing a socket not to wipe out the settings of the 
	 * other socket.
	 */
	if(sock == 0)
	{
		switch(configure->vcc)
		{
			case 0:  /* VCC 0V */
				tps_write((tps_read() & ~TPS_AVCC) | TPS_AVCC0);
				break;
			case 33: /* VCC 3.3V */
				tps_write((tps_read() & ~TPS_AVCC) | TPS_AVCC3);
				break;
			case 50: /* VCC 5V */
				tps_write((tps_read() & ~TPS_AVCC) | TPS_AVCC5);
				break;
			default: /* what's this ? */
				printk(KERN_ERR "%s: bad Vcc %d\n", __FUNCTION__, configure->vcc);
				break;
		}
	}
	else
	{
		switch(configure->vcc)
		{
			case 0:  /* VCC 0V */
				tps_write((tps_read() & ~TPS_BVCC) | TPS_BVCC0);
				break;
			case 33: /* VCC 3.3V */
				tps_write((tps_read() & ~TPS_BVCC) | TPS_BVCC3);
				break;
			case 50: /* VCC 5V */
				tps_write((tps_read() & ~TPS_BVCC) | TPS_BVCC5);
				break;
			default: /* what's this ? */
				printk(KERN_ERR "%s: bad Vcc %d\n", __FUNCTION__, configure->vcc);
				break;
		}
	}

	au_sync_delay(300);

	if(sock == 0)
	{
		if(!configure->reset)
		{
			bcsr->cf0 |=  BCSR_CF0_EN;
			au_sync_delay(300);
			bcsr->cf0 &= ~BCSR_CF0_RST;
			au_sync_delay(100);
		}
		else
		{
			bcsr->cf0 = (bcsr->cf0 & ~BCSR_CF0_EN) | BCSR_CF0_RST;
			au_sync_delay(100);
		}
	}
	else
	{
		if(!configure->reset)
		{
			bcsr->cf1 |=  BCSR_CF1_EN;
			au_sync_delay(300);
			bcsr->cf1 &= ~BCSR_CF1_RST;
			au_sync_delay(100);
		}
		else
		{
			bcsr->cf1 = (bcsr->cf1 & ~BCSR_CF1_EN) | BCSR_CF1_RST;
			au_sync_delay(100);
		}
	}

	return 0;
}

struct pcmcia_low_level au1x00_pcmcia_ops =
{ 
	ekit1100_pcmcia_init,
	ekit1100_pcmcia_shutdown,
	ekit1100_pcmcia_socket_state,
	ekit1100_pcmcia_get_irq_info,
	ekit1100_pcmcia_configure_socket
};
